/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#+
#+     Internet Radio Automation & Encoding Toolkit
#+
#+     Copyright (C) 2018, 2019, 2020, 2021, 2022, 2023, 2024, 2025 
#+     by Kevin C. O'Kane
#+
#+     Kevin C. O'Kane
#+     kc.okane@gmail.com
#+     https://www.cs.uni.edu/~okane
#+     https://threadsafebooks.com/
#+
#+ This program is free software; you can redistribute it and/or modify
#+ it under the terms of the GNU General Public License as published by
#+ the Free Software Foundation; either version 2 of the License, or
#+ (at your option) any later version.
#+
#+ This program is distributed in the hope that it will be useful,
#+ but WITHOUT ANY WARRANTY; without even the implied warranty of
#+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#+ GNU General Public License for more details.
#+
#+ You should have received a copy of the GNU General Public License
#+ along with this program; if not, write to the Free Software
#+ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#+
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

//	January 10, 2025

#include "includes.h"
#include "defines.h"
#include "common-externs.h"
#include "common-fcn-headers.h"
#include "global-extern-variables.h"

#include <vlc/libvlc.h>
#include <vlc/libvlc_renderer_discoverer.h>
#include <vlc/libvlc_media.h>
#include <vlc/libvlc_media_player.h>

#define SLASH '/'

#define TEXT_X 104
#define TEXT_Y 150


void    on_dxb1_clicked(GtkButton *b) { // ok button for text box

	gtk_toggle_button_set_active(insert_text, FALSE);

	close_boxes();

	const gchar *sentry = gtk_entry_get_text( g_dbox5Entry );

	if (strlen(sentry) == 0) {
		Message("Invalid response");
		zippy_says("Invalid response.\n");
		return;
		}

	int i, j, k, pcount = 0;

	gdouble b_trans =  gtk_adjustment_get_value(adjustment13); // text trans
	gdouble start =  gtk_adjustment_get_value(adjustment16); // text_start
	gdouble end =  gtk_adjustment_get_value(adjustment15); // text_end
	gdouble size =  gtk_adjustment_get_value(adjustment14); // text_size
	GdkRGBA rgba;
	gtk_color_button_get_rgba(text_color, &rgba);
	GdkRGBA back_rgba;
	gtk_color_button_get_rgba(back_color, &back_rgba);

	gtk_widget_hide(insert_panel);

	char tmp[16000], filename[4096], filetype[256];
	time_t tmark = time(0);

	char scale[16000];

	for (i = 0; i < (SP->scount); i++) {

		if (programData[i]->play) {

			programData[i]->play = 0;

			pcount++;

//------------------------------------------------------------------
//	ffmpeg -i input.mp4 -vf scale=320:240,setsar=1:1 output.mp4
//------------------------------------------------------------------

			if (gtk_toggle_button_get_active(vid_640x480))
				strcpy(scale, "640x480");

			if (gtk_toggle_button_get_active(vid_720x480))
				strcpy(scale, "720x480");

			if (gtk_toggle_button_get_active(vid_1280x720))
				strcpy(scale, "1280x720");

			if (gtk_toggle_button_get_active(vid_1920x1080))
				strcpy(scale, "1920x1080");

			if (gtk_toggle_button_get_active(vid_3840x2160))
				strcpy(scale, "3840x2160");

			new_name(filename, ProgramArray[i], "txt");

			sprintf(tmp, "ffmpeg -y -loglevel info -i \"%s\" -vf fps=25,"
				"\"drawtext=text=\'%s\':font=\'Times New Roman\':"
				"x='w*%f':y='h*%f':fontsize=%d:fontcolor=#%02x%02x%02x:box=1:"
				"boxcolor=#%02x%02x%02x@%.1f:bordercolor=#%02x%02x%02x:"
				"boxborderw=10\"  -c:a libmp3lame -b:a 128k -c:v libx264 -preset veryslow -crf 20  \"%s\"",

                		ProgramArray[i], 
				sentry, 
				(double) (start / pwidth), 
				(double) (end / pheight), 
				(int) size, 

				(int) (rgba.red * 255), (int) (rgba.green * 255),  
					(int) (rgba.blue * 255), 

				(int) (back_rgba.red * 255), 
					(int) (back_rgba.green * 255),  
					(int) (back_rgba.blue * 255),  // box color

				(int) (back_rgba.red * 255), 
					(int) (back_rgba.green * 255),  
					(int) (back_rgba.blue * 255), // border color

				b_trans, 
				filename);

			int err_stat = ff_run("Text", tmp, i);

			if (err_stat != 0) {
				zippy_says("An error occurred\n");
				sprintf(tmp, "rm \"%s\"", filename);
				system(tmp);
				return;
				}

			printf("Created: %s\n\n", filename);

			}
		}


	if (!pcount) {
		zippy_says("No files selected.\n");
		}

        clear_play();

        on_scan_video_dir_clicked(NULL);

	}


void    on_insert_text_toggled(GtkWidget *w) {

        gint x, y, width, height;

	gtk_widget_hide(crop_adj);

        if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w)) ) {
                gtk_widget_show(insert_panel);  // show dialog
                gtk_widget_show(moving_label);  // show dialog
		gtk_widget_hide(crop_adj);
		gtk_widget_hide(set_fixed);
		gtk_widget_hide(ovl_fixed);
		gtk_widget_hide(opt_panel);

		gtk_widget_hide(draw2);
		gtk_widget_hide(join_group17);
		
	        gtk_toggle_button_set_active(settings_dlg_button, FALSE);
		gtk_toggle_button_set_active(crop_video, FALSE);
		gtk_toggle_button_set_active(overlay_clip, FALSE);
		gtk_toggle_button_set_active(adjust_video, FALSE);
                }
        else {
		gtk_widget_hide(insert_panel);
                gtk_widget_hide(moving_label);  // show dialog

                if (gtk_toggle_button_get_active(frame_next)) {
                        gtk_widget_show(draw2);
                        gtk_widget_show(join_group17);
                        }

		return;
		}

        if (mediaPlayer == NULL) {
                zippy_says("No media player currently in use.\n");
		gtk_toggle_button_set_active(insert_text, FALSE); 
                gtk_widget_hide(moving_label);  // show dialog
                return;
                }

	gtk_toggle_button_set_active(frame_next, TRUE); // vlc_pause();
	while (gtk_events_pending ()) gtk_main_iteration ();

	text_x = 0; text_y = 0;

	gtk_spin_button_set_value (GTK_SPIN_BUTTON(text_start), (gdouble) text_x);
	gtk_spin_button_set_value (GTK_SPIN_BUTTON(text_end), (gdouble) text_y);

	text_x = TEXT_X;
	text_y = TEXT_Y;

	const gchar *sentry = gtk_entry_get_text( g_dbox5Entry );

	GdkRGBA rgba;
	int for_red, for_green, for_blue;

	gtk_color_button_get_rgba(text_color, &rgba);

	for_red = rgba.red * 255; for_green=rgba.green * 255; for_blue=rgba.blue * 255;

	int back_red, back_green, back_blue;

	gtk_color_button_get_rgba(back_color, &rgba);

	back_red = rgba.red * 255; back_green=rgba.green * 255; back_blue=rgba.blue * 255;

	int sz =  gtk_adjustment_get_value(adjustment14); // text_size

	sz = sz * 1000;

	char tmp[1024];
	sprintf(tmp, "<span size='%d' foreground='#%x%x%x' background='#%x%x%x'>%s</span>", 
		sz, 
		for_red, for_green, for_blue, 
		back_red, back_green, back_blue, 
		sentry);

	gtk_label_set_markup(GTK_LABEL(moving_label), tmp);

	gtk_fixed_move (GTK_FIXED(Top1), moving_label, text_x, text_y);


        }


//--------------------------
//	vertical control
//--------------------------

void    on_text_end_value_changed(GtkSpinButton *s) {

        text_y = gtk_spin_button_get_value(GTK_SPIN_BUTTON(text_end)) ;
        text_x = gtk_spin_button_get_value(GTK_SPIN_BUTTON(text_start)) ;

	text_x = text_x + TEXT_X;
	text_y = text_y + TEXT_Y;

	const gchar *sentry = gtk_entry_get_text( g_dbox5Entry );

	int sz =  gtk_adjustment_get_value(adjustment14); // text_size

	sz = sz * 1000;

	GdkRGBA rgba;
	int for_red, for_green, for_blue;

	gtk_color_button_get_rgba(text_color, &rgba);

	for_red = rgba.red * 255; for_green=rgba.green * 255; for_blue=rgba.blue * 255;

	int back_red, back_green, back_blue;

	gtk_color_button_get_rgba(back_color, &rgba);

	back_red = rgba.red * 255; back_green=rgba.green * 255; back_blue=rgba.blue * 255;

	char tmp[1024];
	sprintf(tmp, "<span size='%d' foreground='#%x%x%x' background='#%x%x%x'>%s</span>", 
		sz, 
		for_red, for_green, for_blue, 
		back_red, back_green, back_blue, 
		sentry);

	gtk_label_set_markup(GTK_LABEL(moving_label), tmp);

        gtk_fixed_move (GTK_FIXED(Top1), moving_label, text_x, text_y);

	gtk_widget_show(moving_label);
        }


//--------------------------
//	horizontal control
//--------------------------

void    on_text_start_value_changed(GtkSpinButton *s) {

        text_y = gtk_spin_button_get_value(GTK_SPIN_BUTTON(text_end)) ;
        text_x = gtk_spin_button_get_value(GTK_SPIN_BUTTON(text_start)) ;

	text_x = text_x + TEXT_X;
	text_y = text_y + TEXT_Y;

	const gchar *sentry = gtk_entry_get_text( g_dbox5Entry );

	int sz =  gtk_adjustment_get_value(adjustment14); // text_size

	sz = sz * 1000;

	GdkRGBA rgba;
	int for_red, for_green, for_blue;

	gtk_color_button_get_rgba(text_color, &rgba);

	for_red = rgba.red * 255; for_green=rgba.green * 255; for_blue=rgba.blue * 255;

	int back_red, back_green, back_blue;

	gtk_color_button_get_rgba(back_color, &rgba);

	back_red = rgba.red * 255; back_green=rgba.green * 255; back_blue=rgba.blue * 255;

	char tmp[1024];
	sprintf(tmp, "<span size='%d' foreground='#%x%x%x' background='#%x%x%x'>%s</span>", 
		sz, 
		for_red, for_green, for_blue, 
		back_red, back_green, back_blue, 
		sentry);

	gtk_label_set_markup(GTK_LABEL(moving_label), tmp);

        gtk_fixed_move (GTK_FIXED(Top1), moving_label, text_x, text_y);

	gtk_widget_show(moving_label);
        }

void	on_text_color_color_set(GtkWidget *w) {
	on_text_start_value_changed(NULL); // triggers repaint
	}

void	on_back_color_color_set(GtkWidget *w) {
	on_text_start_value_changed(NULL); // triggers repaint
	}


void	on_text_size_value_changed(GtkWidget *w) { }

